using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using System.Diagnostics;
using System.Threading;

namespace MvcWebRole1
{
    public class WebRole : RoleEntryPoint
    {
        private readonly RemoteDebugManager remoteDebug = new RemoteDebugManager()
        {
            LocalResource = "Tools",
            StorageAccountConfigKey = "Debug.StorageAccount",
            ToolsContainer = "tools",
        };

        public override bool OnStart()
        {
            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.

            RoleEnvironment.Changing += RoleEnvironmentChanging;
            RoleEnvironment.Changed += RoleEnvironmentChanged;

            try
            {
                StartStopDebugService();
            }
            catch (Exception ex)
            {
                //Trace Startup Exception
                Trace.TraceError(ex.Message);

                throw;
            }

            return base.OnStart();
        }

        public override void Run()
        {
            //Simple code to set a breakpoint and demonstrate remote debugging of WaIIsHost process
            int i = 0;
            while (true)
            {
                Thread.Sleep(20000);
                i++;
            }
        }

        private void RoleEnvironmentChanging(object sender, RoleEnvironmentChangingEventArgs e)
        {
            var configChanges = e.Changes.OfType<RoleEnvironmentConfigurationSettingChange>();

            //Basically this simple logic is looking to see if the ONLY configuraiton change for "Debug.Enabled"
            //  if this is the only change then we don't reboot the role/instance.
            //  This logic is likely replaced with role instance settings change logic relevant to the application or somethign more generic
            //  NOTE: We could also handle Debug.HostName changes fairly easily

            //NOTE: There is likely a more efficient way to write this LINQ statement as this will possibly result in multiple enumeration of config changes
            e.Cancel = configChanges.Count() > 1
                       && configChanges.Any(change => change.ConfigurationSettingName == "Debug.Enabled");
        }

        private void RoleEnvironmentChanged(object sender, RoleEnvironmentChangedEventArgs e)
        {
            // If Debug.Enabled has changed then start/stop debug serives
            if (e.Changes.OfType<RoleEnvironmentConfigurationSettingChange>().FirstOrDefault(config => config.ConfigurationSettingName == "Debug.Enabled") != null)
                this.StartStopDebugService();
        }

        //We could alternatively move more of this in to RemoteDebugManager
        private void StartStopDebugService()
        {
            Trace.TraceInformation("StartStopDebugService Method Called");

            //Only enable/disabled when running in actual cloud environment
            if (!RoleEnvironment.IsEmulated)
            {
                this.remoteDebug.InstallAndConfigure();

                //Set HostName
                this.remoteDebug.HostName = CloudConfigurationManager.GetSetting("Debug.HostName");

                var debugEnabled = Boolean.Parse(CloudConfigurationManager.GetSetting("Debug.Enabled"));

                Trace.TraceInformation("StartStopDebugService Called With DebugEnabled: {0}", debugEnabled);

                if (debugEnabled)
                    this.remoteDebug.Start();
                else
                    this.remoteDebug.Stop();
            }
        }
    }
}
